"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.register = exports.createTransformers = void 0;
const transformer_1 = __importDefault(require("./transformer"));
const constants_1 = require("./constants");
function createTransformers(program, configs, oldTransfromers) {
    const { before: oldBefore, after: oldAfter, afterDeclarations: oldAfterDeclarations, ...theRest } = oldTransfromers ?? {};
    const before = [];
    const after = [];
    const afterDeclarations = [];
    configs.forEach(function (config) {
        if (config.afterDeclarations) {
            afterDeclarations.push((0, transformer_1.default)(program, config, null));
        }
        else {
            before.push((0, transformer_1.default)(program, config, null));
        }
    });
    if (oldBefore) {
        before.push(...oldBefore);
    }
    if (oldAfter) {
        after.push(...oldAfter);
    }
    if (oldAfterDeclarations) {
        afterDeclarations.push(...oldAfterDeclarations);
    }
    return {
        before,
        afterDeclarations,
        ...theRest,
    };
}
exports.createTransformers = createTransformers;
function register() {
    // Load ts-node library
    /* eslint-disable-next-line @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-var-requires */
    const tsNode = require('ts-node');
    // Check if ts-node is already loaded
    let tsNodeService = global.process[tsNode.REGISTER_INSTANCE];
    if (!tsNodeService) {
        // If not loaded, create a new instance
        tsNodeService = tsNode.register();
    }
    const transformerConfigs = [];
    const pluginConfigs = tsNodeService.config.options.plugins;
    pluginConfigs?.forEach(function (pluginConfig) {
        // if after=true, the transformer won't be call before the module resolver
        // There is no reason for after to be true
        if (!pluginConfig.after && (pluginConfig.transform === process.env.TRANSFORM_PLUGIN_NAME || constants_1.PROJECT_NAME)) {
            transformerConfigs.push(pluginConfig);
        }
    });
    // Shallow clone register options
    const registerOptions = Object.assign({}, tsNodeService.options);
    if (registerOptions.transformers) {
        // Merge with current transformer
        if (typeof registerOptions.transformers === 'function') {
            const oldTransfomersFactory = registerOptions.transformers;
            // Redifine the transformers
            registerOptions.transformers = (program) => {
                const oldTransformers = oldTransfomersFactory(program);
                return createTransformers(program, transformerConfigs, oldTransformers);
            };
        }
        else {
            const oldTransformers = registerOptions.transformers;
            registerOptions.transformers = function (program) {
                return createTransformers(program, transformerConfigs, oldTransformers);
            };
        }
    }
    else {
        registerOptions.transformers = function (program) {
            return createTransformers(program, transformerConfigs);
        };
    }
    tsNode.register(registerOptions);
    return registerOptions;
}
exports.register = register;
register();
//# sourceMappingURL=register.js.map